==> odamex-src-0.6.4.tar.bz2: odamex-src-0.6.4/common/p_mobj.h <==
// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id: p_mobj.h 3798 2013-04-24 03:09:33Z dr_sean $
//
// Copyright (C) 1993-1996 by id Software, Inc.
// Copyright (C) 2006-2013 by The Odamex Team.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// DESCRIPTION:
//   Map Objects, MObj, definition and handling.
//
//-----------------------------------------------------------------------------


#ifndef __PMOBJ_H__
#define __PMOBJ_H__

#define REMOVECORPSESTIC TICRATE*80

//-----------------------------------------------------------------------------
//
// denis - superior NetIDHandler
//
// Very simple, very fast.
// Does not iterate when releasing a netid.
// Does not iterate when obtaining a netid unless all pooled ids are taken.
// (in which case does one allocation and does not iterate more than 
//  MEMPOOLSIZE times)
// Only downside is that it won't detect double-releasing, but shouldn't be 
// a problem.
//
// Thanks to [Dash|RD] for noticing the efficiency problem, hard work on other 
// versions of this class and giving me this great idea.
//
//-----------------------------------------------------------------------------

// [SL] 2012-04-04 
// Modified to use a std::queue, popping from the front of the queue to assign
// new netids and pushing newly freed netids on the back of the queue.  This is
// to avoid reassigning a recently freed netid to a different actor.  Otherwise
// clients can get confused when packets are dropped.

#include "i_system.h"
#include <queue>

#define MAX_NETID 0xFFFF

class NetIDHandler
{
	private:

	size_t NumAllocated;
	std::queue<int> free_ids;
	const size_t ChunkSize;

	public:

	NetIDHandler(size_t chunk_size = 512)
		: NumAllocated(0), ChunkSize(chunk_size)
	{}

	~NetIDHandler()
	{
	}

	int ObtainNetID()
	{
		if (free_ids.empty())
		{
			if(NumAllocated >= MAX_NETID - 1)
				I_Error("Exceeded maximum number of netids");

			int OldAllocated = NumAllocated;
			NumAllocated += ChunkSize;

			if(NumAllocated >= MAX_NETID - 1)
				NumAllocated = MAX_NETID - 1;

			for (size_t i = OldAllocated + 1; i <= NumAllocated; i++)
				free_ids.push(i);
		}

		int netid = free_ids.front();
		free_ids.pop();
	
		return netid;
	}

	void ReleaseNetID(int NetID)
	{
		if (!NetID || NetID > (int)NumAllocated)
			I_Error("Released a non-existant netid %d", NetID);

		free_ids.push(NetID);
	}
};

extern NetIDHandler ServerNetID;

void P_ClearAllNetIds();
AActor* P_FindThingById(size_t id);
void P_SetThingId(AActor *mo, size_t newnetid);
void P_ClearId(size_t id);

bool P_SetMobjState(AActor *mobj, statenum_t state, bool cl_update);
void P_XYMovement(AActor *mo);
void P_ZMovement(AActor *mo);
void PlayerLandedOnThing(AActor *mo, AActor *onmobj); // [CG] Used to be 'static'
void P_NightmareRespawn(AActor *mo);
void P_SpawnPuff(fixed_t x, fixed_t y, fixed_t z, angle_t dir, int updown);
void P_SpawnBlood(fixed_t x, fixed_t y, fixed_t z, angle_t dir, int damage);
bool P_CheckMissileSpawn(AActor* th);
AActor* P_SpawnMissile(AActor *source, AActor *dest, mobjtype_t type);
void P_SpawnPlayerMissile(AActor *source, mobjtype_t type);
bool P_VisibleToPlayers(AActor *mo);

// [ML] From EE
int P_ThingInfoHeight(mobjinfo_t *mi);

#endif

==> odamex-src-0.9.5.tar.bz2: odamex-src-0.9.5/common/p_mobj.h <==
// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id: 688a8864c6f3d6ea478a30269a326e661ab42e66 $
//
// Copyright (C) 1993-1996 by id Software, Inc.
// Copyright (C) 2006-2020 by The Odamex Team.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// DESCRIPTION:
//   Map Objects, MObj, definition and handling.
//
//-----------------------------------------------------------------------------


#ifndef __PMOBJ_H__
#define __PMOBJ_H__

#define REMOVECORPSESTIC TICRATE*80

//-----------------------------------------------------------------------------
//
// denis - superior NetIDHandler
//
// Very simple, very fast.
// Does not iterate when releasing a netid.
// Does not iterate when obtaining a netid unless all pooled ids are taken.
// (in which case does one allocation and does not iterate more than 
//  MEMPOOLSIZE times)
// Only downside is that it won't detect double-releasing, but shouldn't be 
// a problem.
//
// Thanks to [Dash|RD] for noticing the efficiency problem, hard work on other 
// versions of this class and giving me this great idea.
//
//-----------------------------------------------------------------------------

// [SL] 2012-04-04 
// Modified to use a std::queue, popping from the front of the queue to assign
// new netids and pushing newly freed netids on the back of the queue.  This is
// to avoid reassigning a recently freed netid to a different actor.  Otherwise
// clients can get confused when packets are dropped.

// [AM] 2021-03-05
// Changed NetID system to not "give back" NetID's.  This was causing issues
// when resetting the level too many times.

#include "i_system.h"

#define MAX_NETID 0xFFFFFFFF

class NetIDHandler
{
  private:
	uint32_t m_nextID;

  public:
	NetIDHandler() : m_nextID(1)
	{
	}

	~NetIDHandler()
	{
	}

	uint32_t peekNetID()
	{
		return m_nextID;
	}

	/**
	 * @brief Obtain a netID for an AActor.
	 */
	uint32_t obtainNetID()
	{
		if (m_nextID == MAX_NETID)
		{
			I_Error("Exceeded maximum number of netids (%u)", MAX_NETID);
		}

		m_nextID += 1;
		return m_nextID - 1;
	}

	/**
	 * @brief Reset the netID back to 1.
	 *
	 * @detail Probably not a good idea to call this method in the middle
	 *         of a level, reusing netID's doesn't tend to go very well.
	 */
	void resetNetIDs()
	{
		m_nextID = 1;
	}
};

extern NetIDHandler ServerNetID;

void P_ClearAllNetIds();
AActor* P_FindThingById(uint32_t id);
void P_SetThingId(AActor* mo, uint32_t newnetid);
void P_ClearId(uint32_t id);

bool P_SetMobjState(AActor *mobj, statenum_t state, bool cl_update);
void P_XYMovement(AActor *mo);
void P_ZMovement(AActor *mo);
void PlayerLandedOnThing(AActor *mo, AActor *onmobj); // [CG] Used to be 'static'
void P_NightmareRespawn(AActor *mo);
void P_SpawnPuff(fixed_t x, fixed_t y, fixed_t z);
void P_SpawnTracerPuff(fixed_t x, fixed_t y, fixed_t z);
void P_SpawnBlood(fixed_t x, fixed_t y, fixed_t z, angle_t dir, int damage);
bool P_CheckMissileSpawn(AActor* th);
AActor* P_SpawnMissile(AActor *source, AActor *dest, mobjtype_t type);
void P_SpawnPlayerMissile(AActor *source, mobjtype_t type);
bool P_VisibleToPlayers(AActor *mo);

// [ML] From EE
int P_ThingInfoHeight(mobjinfo_t *mi);
void SpawnFlag(mapthing2_t* mthing, team_t flag);

#endif
